/*
 * Cr le 12 fvr. 2004
 *
 * ViewerJava3D
 */

import java.awt.*;
import javax.swing.JPanel;
import javax.media.j3d.*;
import javax.vecmath.*;
import com.sun.j3d.utils.geometry.*;
import com.sun.j3d.utils.behaviors.keyboard.*;
import com.sun.j3d.utils.behaviors.mouse.*;

/**
 * @author Olivier Lamotte
 *
 * Cration d'un viewer en redfinissant les branches visualisation et objet
 */
public class ViewerJava3D extends JPanel
{
	
	private static final long serialVersionUID = 0;
	
	/** BranchGroup de la scne */
	private TransformGroup contentTransformGroup = null;
	
	/** Group du sceneGraph */
	private Group sceneGroup = null;
	private Group gridGroup = null;
	
	/** Affichage de la grille */
	public boolean gridVisible = true;
	
	public static final float DEFAULT_TRANSPARENCY = 0.5f;
	
	/**
	 * Constructeur
	 */
	public ViewerJava3D()
	{
		// Initialisation du JPanel
		super();
		setLayout(new BorderLayout());
		
		//====================================
		
		// Cration du contexte graphique
		GraphicsEnvironment environment = GraphicsEnvironment.getLocalGraphicsEnvironment();
		GraphicsDevice device = environment.getDefaultScreenDevice();
		GraphicsConfigTemplate3D configTemplate = new GraphicsConfigTemplate3D();
		GraphicsConfiguration config = device.getBestConfiguration(configTemplate);
		
		// Cration du canvas
		Canvas3D canvas = new Canvas3D(config);
		
		// Ajout du Canvas3D au centre du JPanel
		//add(canvas);
		add(BorderLayout.CENTER, canvas);
		
		// Cration de l'univers
		VirtualUniverse universe = new VirtualUniverse();
		
		// Cration du Locale
		Locale locale = new Locale(universe);
		
		// Cration et ajout de la branche de visualisation
		locale.addBranchGraph(createViewBranch(canvas));
		
		// Cration et ajout de la branche objet
		locale.addBranchGraph(createObjectBranch());
}
	
	/**
	 * createViewBranch
	 *
	 * Cration de la branche de visualisation
	 * @param canvas
	 * @return BranchGroup visualisation
	 */	
	private BranchGroup createViewBranch(Canvas3D canvas)
	{
		// Cration de la banche View
		BranchGroup viewBranch = new BranchGroup();
		
		/*
		 
		 // Positionnement du point de vue
		  Transform3D transform = new Transform3D();
		  transform.setTranslation(new Vector3d(-10,0,0));
		  
		  // Cration du TransformGroup View
		   TransformGroup viewTransformGroup = new TransformGroup(transform);
		   viewBranch.addChild(viewTransformGroup);
		   
		   // Cration du ViewPlatform
		    ViewPlatform viewPlatform = new ViewPlatform();
		    viewTransformGroup.addChild(viewPlatform);
		    
		    // Cration de la vue
		     View view = new View();
		     view.attachViewPlatform(viewPlatform);
		     
		     // Constitution de la branche visualisation (Attention  l'ordre des tapes)
		      view.addCanvas3D(canvas);
		      view.setPhysicalBody(new PhysicalBody());
		      view.setPhysicalEnvironment(new PhysicalEnvironment());
		      
		      */
		
		// Positionnement du point de vue
		Transform3D viewPointTranslation = new Transform3D();
		Transform3D viewPointRotation = new Transform3D();
		
		viewPointTranslation.set(new Vector3f(0.0f, 1.5f, 10.0f));
		viewPointRotation.rotX(-0.2);
		
		// Composition des transformations
		viewPointTranslation.mul(viewPointRotation);
		
		// Cration du TransformGroup View
		TransformGroup viewTransformGroup = new TransformGroup(viewPointTranslation);
		viewTransformGroup.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
		viewTransformGroup.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
		
		// Mise en place des clippings (proche et distant)
		// nb : Clipping proche = pour compenser l'absence de stro (une seule
		// camra)
		
		// Cration du ViewPlatform
		ViewPlatform viewPlatform = new ViewPlatform();
		
		// Cration de la vue
		View view = new View();
		view.setBackClipDistance(100000);
		view.attachViewPlatform(viewPlatform);
		view.addCanvas3D(canvas);
		view.setPhysicalBody(new PhysicalBody());
		view.setPhysicalEnvironment(new PhysicalEnvironment());
		
		// Navigation
		KeyNavigatorBehavior keys = new KeyNavigatorBehavior(viewTransformGroup);
		keys.setSchedulingBounds(new BoundingBox());
		//keys.initialize();
		MouseRotate mouseR = new MouseRotate(viewTransformGroup);
		keys.setSchedulingBounds(new BoundingBox());
		//mouseR.initialize();
		//MouseTranslate mouseT = new MouseTranslate(viewTransformGroup);
		//MouseZoom mouseZ = new MouseZoom(viewTransformGroup);
		
		// Constitution de la branche visualisation (Attention  l'ordre des tapes)
		viewTransformGroup.addChild(viewPlatform);
		viewTransformGroup.addChild(keys);
		viewTransformGroup.addChild(mouseR);
		//viewTransformGroup.addChild(mouseT);
		//viewTransformGroup.addChild(mouseZ);
		viewBranch.addChild(viewTransformGroup);
		viewBranch.compile();
		
		return (viewBranch);
	}
	
	/**
	 * createObjectBranch
	 * 
	 * Cration de la branche contenant les objets
	 * @return BranchGroup contenant les objets
	 */
	private BranchGroup createObjectBranch()
	{
		// Cration de la contentBranch
		BranchGroup contentBranch = new BranchGroup();
		
		// Cration du TransformGroup content
		//TransformGroup contentTransformGroup = new TranformGroup();
		contentTransformGroup = new TransformGroup();
		contentBranch.addChild(contentTransformGroup);
		
		// Cration du groupe scne
		//BranchGroup scene = createSceneGraph()
		sceneGroup = new BranchGroup();
		
		// Cration de la Scene
		// --------------------
		createScene();
		
		// Constitution de la branche scne
		// --------------------------------
		contentTransformGroup.addChild(sceneGroup);
		
		// Optimisation des calculs
		//contentBranch.compile();
		
		return contentBranch;
	}
	
	/**
	 * createScene
	 *
	 * Cration de la scne  afficher
	 */
	//private void createScene(TransformGroup tgScene)
	private void createScene()
	{
		
		//========== Cube =================
		
		ColorCube cube = new ColorCube(0.2);
		
		TransformGroup tg1 = new TransformGroup();
		tg1.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
		sceneGroup.addChild(tg1);
		
		Transform3D tx1 = new Transform3D();
		tx1.rotX(Math.PI/2);
		RotationInterpolator rx1 = new RotationInterpolator(
				new Alpha(-1,2000), tg1, tx1,
				0.0f,(float)(2*Math.PI)
		);
		rx1.setSchedulingBounds(new BoundingBox());
		tg1.addChild(rx1);
		
		TransformGroup tg2 = new TransformGroup();
		tg2.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
		
		Transform3D tx2 = new Transform3D();
		tx2.rotY(Math.PI/2);
		RotationInterpolator rx2 = new RotationInterpolator(
				new Alpha(-1,2000), tg2, tx2,
				0.0f,(float)(2*Math.PI)
		);
		rx2.setSchedulingBounds(new BoundingBox());
		tg1.addChild(tg2);
		tg2.addChild(rx2);
		
		// tg1->tg2->cube
		// Nb : on aurait aussi pu multiplier les Transform3D
		
		tg2.addChild(cube);
		
		//========== Grilles =================
		
		gridGroup = new BranchGroup();
		gridGroup.setCapability(BranchGroup.ALLOW_CHILDREN_READ);
		sceneGroup.addChild(gridGroup);
		
		gridGroup.addChild(buildGrid('x','y'));
		gridGroup.addChild(buildGrid('y','z'));
		gridGroup.addChild(buildGrid('z','x'));
		
	}
	
	private Shape3D buildGrid(char axis1, char axis2)
	{

		int bndX=0,varX=0,bndY=0,varY=0,bndZ=0,varZ=0;

		switch(axis1)
		{
		case 'x' : varX=1; break;
		case 'y' : varY=1; break;
		case 'z' : varZ=1; break;
		default : return null;
		}
		switch(axis2)
		{
		case 'x' : bndX=1; break;
		case 'y' : bndY=1; break;
		case 'z' : bndZ=1; break;
		default : return null;
		}
		
		int cpt;
		Point3f[] coordinates;
		float var;
		
		// Grille //  axis1
		coordinates = new Point3f[84];
		cpt=0;
		var=-1;
		while (cpt<42)
		{
			coordinates[cpt++] = new Point3f(-1*bndX+var*varX,-1*bndY+var*varY,-1*bndZ+var*varZ);
			coordinates[cpt++] = new Point3f(1*bndX+var*varX,1*bndY+var*varY,1*bndZ+var*varZ);
			var+=0.1f;
		}
		// Grille //  axis2
		var=-1;
		while (cpt<84)
		{
			coordinates[cpt++] = new Point3f(-1*varX+var*bndX,-1*varY+var*bndY,-1*varZ+var*bndZ);
			coordinates[cpt++] = new Point3f(1*varX+var*bndX,1*varY+var*bndY,1*varZ+var*bndZ);
			var+=0.1f;
		}
		LineArray gridGeometry = new LineArray(84,LineArray.COORDINATES);
		gridGeometry.setCoordinates(0,coordinates);
		
		Appearance gridAppearance = new Appearance();
		gridAppearance.setCapability(Appearance.ALLOW_TRANSPARENCY_ATTRIBUTES_WRITE);
		gridAppearance.setTransparencyAttributes(new TransparencyAttributes(
				TransparencyAttributes.FASTEST,ViewerJava3D.DEFAULT_TRANSPARENCY));
		
		Shape3D grid = new Shape3D(gridGeometry,gridAppearance);
		grid.setCapability(Shape3D.ALLOW_APPEARANCE_WRITE);
		
		return grid;
	}
		
	public void switchGrid()
	{
		float trans;
		if (gridVisible == true) trans = 1.0f;
		else trans = ViewerJava3D.DEFAULT_TRANSPARENCY;
		gridVisible = !gridVisible;

		Appearance gridAppearance = new Appearance();
		gridAppearance.setCapability(Appearance.ALLOW_TRANSPARENCY_ATTRIBUTES_WRITE);
		gridAppearance.setTransparencyAttributes(new TransparencyAttributes(
				TransparencyAttributes.FASTEST,trans));		
		
		for (int i=0; i<2; i++)
			((Shape3D)gridGroup.getChild(i)).setAppearance(gridAppearance);
	}
}