/*
 * Cr le 12 fvr. 2004
 *
 * ViewerJava3D
 */

import java.awt.*;
import javax.swing.JPanel;
import javax.media.j3d.*;
import javax.vecmath.*;
import com.sun.j3d.utils.geometry.*;
import com.sun.j3d.utils.behaviors.keyboard.*;
import com.sun.j3d.utils.behaviors.mouse.*;
import com.sun.j3d.utils.image.TextureLoader;

/**
 * @author Olivier Lamotte
 *
 * Cration d'un viewer en redfinissant les branches visualisation et objet
 */
public class ViewerJava3D extends JPanel
{
	
	private static final long serialVersionUID = 0;
	
	/** BranchGroup de la scne */
	private TransformGroup contentTransformGroup = null;
	
	/** Group du sceneGraph */
	private Group sceneGroup = null;
	
	/** Affichage de la grille */
	public boolean gridVisible = true;
	
	public static final float DEFAULT_TRANSPARENCY = 0.5f;
	
	/**
	 * Constructeur
	 */
	public ViewerJava3D()
	{
		// Initialisation du JPanel
		super();
		setLayout(new BorderLayout());
		
		//====================================
		
		// Cration du contexte graphique
		GraphicsEnvironment environment = GraphicsEnvironment.getLocalGraphicsEnvironment();
		GraphicsDevice device = environment.getDefaultScreenDevice();
		GraphicsConfigTemplate3D configTemplate = new GraphicsConfigTemplate3D();
		GraphicsConfiguration config = device.getBestConfiguration(configTemplate);
		
		// Cration du canvas
		Canvas3D canvas = new Canvas3D(config);
		
		// Ajout du Canvas3D au centre du JPanel
		//add(canvas);
		add(BorderLayout.CENTER, canvas);
		
		// Cration de l'univers
		VirtualUniverse universe = new VirtualUniverse();
		
		// Cration du Locale
		Locale locale = new Locale(universe);
		
		// Cration et ajout de la branche de visualisation
		locale.addBranchGraph(createViewBranch(canvas));
		
		// Cration et ajout de la branche objet
		locale.addBranchGraph(createObjectBranch());
}
	
	/**
	 * createViewBranch
	 *
	 * Cration de la branche de visualisation
	 * @param canvas
	 * @return BranchGroup visualisation
	 */	
	private BranchGroup createViewBranch(Canvas3D canvas)
	{
		// Cration de la banche View
		BranchGroup viewBranch = new BranchGroup();
		
		/*
		 
		 // Positionnement du point de vue
		  Transform3D transform = new Transform3D();
		  transform.setTranslation(new Vector3d(-10,0,0));
		  
		  // Cration du TransformGroup View
		   TransformGroup viewTransformGroup = new TransformGroup(transform);
		   viewBranch.addChild(viewTransformGroup);
		   
		   // Cration du ViewPlatform
		    ViewPlatform viewPlatform = new ViewPlatform();
		    viewTransformGroup.addChild(viewPlatform);
		    
		    // Cration de la vue
		     View view = new View();
		     view.attachViewPlatform(viewPlatform);
		     
		     // Constitution de la branche visualisation (Attention  l'ordre des tapes)
		      view.addCanvas3D(canvas);
		      view.setPhysicalBody(new PhysicalBody());
		      view.setPhysicalEnvironment(new PhysicalEnvironment());
		      
		      */
		
		// Positionnement du point de vue
		Transform3D viewPointTranslation = new Transform3D();
		Transform3D viewPointRotation = new Transform3D();
		
		viewPointTranslation.set(new Vector3f(0.0f, 1.5f, 10.0f));
		viewPointRotation.rotX(-0.2);
		
		// Composition des transformations
		viewPointTranslation.mul(viewPointRotation);
		
		// Cration du TransformGroup View
		TransformGroup viewTransformGroup = new TransformGroup(viewPointTranslation);
		viewTransformGroup.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
		viewTransformGroup.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
		
		// Mise en place des clippings (proche et distant)
		// nb : Clipping proche = pour compenser l'absence de stro (une seule
		// camra)
		
		// Cration du ViewPlatform
		ViewPlatform viewPlatform = new ViewPlatform();
		
		// Cration de la vue
		View view = new View();
		view.setBackClipDistance(100000);
		view.attachViewPlatform(viewPlatform);
		view.addCanvas3D(canvas);
		view.setPhysicalBody(new PhysicalBody());
		view.setPhysicalEnvironment(new PhysicalEnvironment());
		
		// Navigation
		KeyNavigatorBehavior keys = new KeyNavigatorBehavior(viewTransformGroup);
		keys.setSchedulingBounds(new BoundingBox());
		//keys.initialize();
		MouseRotate mouseR = new MouseRotate(viewTransformGroup);
		keys.setSchedulingBounds(new BoundingBox());
		//mouseR.initialize();
		//MouseTranslate mouseT = new MouseTranslate(viewTransformGroup);
		//MouseZoom mouseZ = new MouseZoom(viewTransformGroup);
		
		// Constitution de la branche visualisation (Attention  l'ordre des tapes)
		viewTransformGroup.addChild(viewPlatform);
		viewTransformGroup.addChild(keys);
		viewBranch.addChild(mouseR);
		//viewTransformGroup.addChild(mouseT);
		//viewTransformGroup.addChild(mouseZ);
		viewBranch.addChild(viewTransformGroup);
		viewBranch.compile();
		
		return (viewBranch);
	}
	
	/**
	 * createObjectBranch
	 * 
	 * Cration de la branche contenant les objets
	 * @return BranchGroup contenant les objets
	 */
	private BranchGroup createObjectBranch()
	{
		// Cration de la contentBranch
		BranchGroup contentBranch = new BranchGroup();
		
		// Cration du TransformGroup content
		//TransformGroup contentTransformGroup = new TranformGroup();
		contentTransformGroup = new TransformGroup();
		contentBranch.addChild(contentTransformGroup);
		
		// Cration du groupe scne
		//BranchGroup scene = createSceneGraph()
		sceneGroup = new BranchGroup();
		
		// Cration de la Scene
		// --------------------
		createScene();
		
		// Constitution de la branche scne
		// --------------------------------
		contentTransformGroup.addChild(sceneGroup);
		
		// Optimisation des calculs
		//contentBranch.compile();
		
		return contentBranch;
	}
	
	/**
	 * createScene
	 *
	 * Cration de la scne  afficher
	 */
	//private void createScene(TransformGroup tgScene)
	private void createScene()
	{
		
		Appearance sunAppearance = new Appearance();
		Material sunMaterial = new Material();
		sunMaterial.setCapability(Material.ALLOW_COMPONENT_WRITE);
		sunMaterial.setEmissiveColor(1.0f,1.0f,0.0f);
		sunMaterial.setLightingEnable(true);
		sunAppearance.setMaterial(sunMaterial);
		Sphere sun = new Sphere(0.25f,sunAppearance);

		// Eclairage
		AmbientLight sunLight = 
			new AmbientLight(new Color3f(.8f,.8f,.8f));
		sunLight.setInfluencingBounds(new BoundingSphere(new Point3d(0,0,0),10.0f));
		
		sceneGroup.addChild(sun);
		sceneGroup.addChild(sunLight);		

		// Y axis
		Transform3D rot = new Transform3D();
		rot.rotY(Math.PI/2);		
		
		TransformGroup sunGroup = new TransformGroup();
		sunGroup.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
		sunGroup.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
		RotationInterpolator sunInterp = new RotationInterpolator(
				new Alpha(-1,36500), sunGroup, rot,
				0.0f,(float)(2*Math.PI));
		sunInterp.setSchedulingBounds(new BoundingSphere());
		sunGroup.addChild(sunInterp);/**/		
		
		Transform3D earthTrans = new Transform3D();
		earthTrans.setTranslation(new Vector3d(3.0f,0.0f,0.0f));
		TransformGroup earthOrbit = new TransformGroup(earthTrans);

		TransformGroup earthAxis = new TransformGroup();
		earthAxis.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
		earthAxis.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
		RotationInterpolator earthAutoInterp = new RotationInterpolator(
				new Alpha(-1,1000), earthAxis, rot,
				0.0f,(float)(2*Math.PI));
		earthAutoInterp.setSchedulingBounds(new BoundingSphere());
		earthAxis.addChild(earthAutoInterp);/**/		
		
		earthOrbit.addChild(earthAxis);
		sunGroup.addChild(earthOrbit);
		sun.addChild(sunGroup);
		
		Appearance earthAppearance = new Appearance();
		try
		{
			// chargement d'une image pour texture
			TextureLoader loader = new TextureLoader("worldmap.jpg",this);
			ImageComponent2D image = loader.getImage();
			// instanciation de l'objet texture
			Texture2D texture = new Texture2D(Texture.BASE_LEVEL,Texture.RGBA,image.getWidth(),image.getHeight());
			texture.setImage(0, image);
			// ajout de la texture  l'apparence
			earthAppearance.setTexture(texture);
			earthAppearance.setTextureAttributes(new TextureAttributes());
		}
		catch(NullPointerException e1)
		{
			// Creation d'une texture uni dans le cas ou le fichier image n'exciste pas
			ColoringAttributes bleu = new ColoringAttributes();
			bleu.setColor(0.0f,0.0f,0.8f);
			earthAppearance.setColoringAttributes(bleu);
		}		
		Sphere earth = new Sphere(0.15f,Sphere.GENERATE_TEXTURE_COORDS,earthAppearance);//(0.2f,earthAppearance);
		
		earthAxis.addChild(earth);
		
		Appearance moonAppearance = new Appearance();
		Material moonMaterial = new Material();
		moonMaterial.setCapability(Material.ALLOW_COMPONENT_WRITE);
		moonMaterial.setAmbientColor(0.5f,0.5f,0.5f);
		moonAppearance.setMaterial(moonMaterial);
		Sphere moon = new Sphere(0.07f,moonAppearance);
		
		TransformGroup earthGroup = new TransformGroup();
		earthGroup.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
		earthGroup.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
		RotationInterpolator earthInterp = new RotationInterpolator(
				new Alpha(-1,2400), earthGroup, rot,
				0.0f,(float)(2*Math.PI));
		earthInterp.setSchedulingBounds(new BoundingSphere());
		earthGroup.addChild(earthInterp);/**/		
		Transform3D moonTrans = new Transform3D();
		moonTrans.setTranslation(new Vector3d(0.5f,0.0f,0.0f));
		TransformGroup moonOrbit = new TransformGroup(moonTrans);
		
		moonOrbit.addChild(moon);
		earthGroup.addChild(moonOrbit);
		earthOrbit.addChild(earthGroup);
				
	}
	
}